//=============================================================================
// DataManager.cpp
//=============================================================================
#include "Stdinc.h"
#include "DataManager.h"
#include "MapManager.h"
#include "MapObject.h"
#include "MapEvent.h"
#include "xml/tinyxml.h"

const MapDataInfo MAP_DATA_INFO[] = 
{
	{ 000001, 	// }bv1
		TRUE,
		{ 000001, 000001, 000001 },
	},
	{ 000002,	// }bv2
		FALSE,
		{  10002,  10002,  10002 },
	},
};
//-----------------------------------------------------------------------------
// RXgN^
//-----------------------------------------------------------------------------
CDataManager::CDataManager(void)
:m_bLoadedMapChipGraph( FALSE ), m_bLoadedCharacterGraph( FALSE )
{
}

//-----------------------------------------------------------------------------
// fXgN^
//-----------------------------------------------------------------------------
CDataManager::~CDataManager(void)
{

}
//-----------------------------------------------------------------------------
// GetInstance
//-----------------------------------------------------------------------------
CDataManager *CDataManager::GetInstance( void )
{
	static CDataManager m_Instance;
	return &m_Instance;
}
//-----------------------------------------------------------------------------
// LoadMap
//-----------------------------------------------------------------------------
BOOL CDataManager::LoadMap( int nMapID, CMapManager *pMap )
{
	char szMapFileName[MAX_PATH];
	RtlZeroMemory( szMapFileName, sizeof(szMapFileName) );


	// IDΉ }bv擾
	int nChipID[LAYER_CHIP_MAX];	// CChipG
	int nIndex = 0;
	for ( int iCnt = 0; iCnt < sizeof(MAP_DATA_INFO) / sizeof(MAP_DATA_INFO[0]); iCnt ++ )
	{
		if ( MAP_DATA_INFO[iCnt].nMapID == nMapID )
		{
			nIndex = iCnt;
			// }bvt@C
			sprintf_s( szMapFileName,	 "%s\\map%06d.dat",  MAP_DATA_DIR, MAP_DATA_INFO[iCnt].nMapID );
			// }bv`bvID
			memcpy_s( nChipID, sizeof(nChipID),  MAP_DATA_INFO[iCnt].nChipID, sizeof(nChipID) );
			break;
		}
	}

	// }bvt@CǍ
	int nFileHandle = FileRead_open( szMapFileName );
	if ( nFileHandle == DX_INVALID_HANDLE )
	{
		return FALSE;
	}

	// wb_Ǎ
	FMFHEADER fmfHeader;
	if ( FileRead_read( &fmfHeader, sizeof(fmfHeader), nFileHandle ) == DX_FAILED )
	{
		FileRead_close( nFileHandle );
		return FALSE;
	}

	// t@C
	if ( fmfHeader.byLayerCount != LAYER_MAX )
	{
		FileRead_close( nFileHandle );
		return FALSE;
	}

	// }bvݒ
	pMap->SetMapInfo( fmfHeader.dwWidth, fmfHeader.dwHeight, nChipID, MAP_DATA_INFO[nIndex].bLoopAtEdge );

	// `bv̈m
	WORD **pMapChip = pMap->GetMapChip();
	int iReadCnt = fmfHeader.dwWidth * fmfHeader.dwHeight;
	for ( int iLayer = 0; iLayer < LAYER_CHIP_MAX; iLayer ++ )
	{
		pMapChip[iLayer] = new WORD[iReadCnt];
		RtlZeroMemory( pMapChip[iLayer], sizeof(WORD) * iReadCnt );
		if ( FileRead_read( pMapChip[iLayer], sizeof(WORD) * iReadCnt, nFileHandle ) == DX_FAILED )
		{
			FileRead_close( nFileHandle );
			return FALSE;
		}
	}

	// ʍs/s擾
	pMap->m_wPassFlag = new WORD[iReadCnt];
	RtlZeroMemory( pMap->m_wPassFlag, sizeof(WORD) * iReadCnt );
	if ( FileRead_read( pMap->m_wPassFlag, sizeof(WORD) * iReadCnt, nFileHandle ) == DX_FAILED )
	{
		FileRead_close( nFileHandle );
		return FALSE;
	}

	// }bvIuWFNg擾
	pMap->m_MapEvent.m_wMapObject = new WORD[iReadCnt];
	RtlZeroMemory( pMap->m_MapEvent.m_wMapObject, sizeof(WORD) * iReadCnt );
	if ( FileRead_read( pMap->m_MapEvent.m_wMapObject, sizeof(WORD) * iReadCnt, nFileHandle ) == DX_FAILED )
	{
		FileRead_close( nFileHandle );
		return FALSE;
	}

	FileRead_close( nFileHandle );

	return TRUE;
}


//-----------------------------------------------------------------------------
// LoadMapChipGraph
// S}bv`bvǍ
//-----------------------------------------------------------------------------
BOOL CDataManager::LoadMapChipGraph( std::map<int, int> &mapInfo )
{
	if ( m_bLoadedMapChipGraph )
	{
		return TRUE;
	}

	using namespace std;

	char szFileName[MAX_PATH];
	for ( int iCnt = 0; iCnt < MAP_CHIP_GRAPH_MAX; iCnt ++ )
	{
		sprintf_s( szFileName, sizeof(szFileName), "%s\\chip%06d.bmp", MAP_DATA_DIR, MAP_CHIP_ID[iCnt] );

		// }bv`bvGǂݍ
		SetTransColor(0,117,117);
		m_nHandleMapChip[iCnt] = ::LoadGraph( szFileName );
		if ( m_nHandleMapChip[iCnt] == DX_FAILED )
		{
			CSysError::Msg("摜ǂݍݎs [%s]", szFileName );
			return FALSE;
		}

		mapInfo.insert( pair<int, int>( MAP_CHIP_ID[iCnt], m_nHandleMapChip[iCnt] ));
	}

	m_bLoadedMapChipGraph = TRUE;

	return TRUE;
}
//-----------------------------------------------------------------------------
// DeleteMapChipGraph
// S}bv`bv폜
//-----------------------------------------------------------------------------
void CDataManager::DeleteMapChipGraph( void )
{
	for ( int iCnt = 0; iCnt < MAP_CHIP_GRAPH_MAX; iCnt ++ )
	{
		DeleteGraph( m_nHandleMapChip[iCnt] );
	}
	m_bLoadedMapChipGraph = FALSE;

}
//-----------------------------------------------------------------------------
// LoadCharacterGraph
// SLN^GǍ
//-----------------------------------------------------------------------------
BOOL CDataManager::LoadCharacterGraph( void )
{
	if ( m_bLoadedCharacterGraph )
	{
		return FALSE;
	}

	// ToDo: 1ǂłȂ
	char szFileName[MAX_PATH];
	sprintf_s( szFileName, sizeof(szFileName), "%s\\%s", CHAR_DATA_DIR, CHAR_GRAPH_FILENAME );

	// Gǂݍ
	SetTransColor(0,117,117);
	m_nHandleCharacter[0] = ::LoadGraph( szFileName );
	if ( m_nHandleCharacter[0] == DX_FAILED )
	{
		CSysError::Msg("摜ǂݍݎs [%s]", szFileName );
		return FALSE;
	}

	m_bLoadedCharacterGraph = TRUE;

	return TRUE;
}
//-----------------------------------------------------------------------------
// DeleteMapChipGraph
// SLN^G폜
//-----------------------------------------------------------------------------
void CDataManager::DeleteCharacterGraph( void )
{
	for ( int iCnt = 0; iCnt < CHAR_GRAPH_MAX; iCnt ++ )
	{
		DeleteGraph( m_nHandleCharacter[iCnt] );
	}

	m_bLoadedCharacterGraph = FALSE;

}

// for test
static void p(TiXmlElement* e)
{
	if(e == NULL)
		return;
	for(e ; e ; e = e->NextSiblingElement())
	{
		printfDx("%s ", e->Value() );

		const char* text = e->GetText();
		if(text != NULL)
		{
			printfDx(": %s ", text );
		}
		printfDx("\n");

		// Agr[g̎擾
		TiXmlAttribute* a = e->FirstAttribute();
		for(;;)
		{
			if(a == NULL)
				break;
			if(a != NULL)
			{
				printfDx("attribute :  %s ", a->Name() );
				printfDx(" %s\n", a->Value() );
			}
			a = a->Next();
			if( a == NULL)
				break;
		}

		p(e->FirstChildElement());
	}
}
//-----------------------------------------------------------------------------
// LoadMapEvent
// ToDo : XMLp[Tt@C
//-----------------------------------------------------------------------------
#pragma warning(disable : 4996)
BOOL CDataManager::LoadMapEvent( int nMapID, CMapEvent *pMapEvent )
{
	char szFileName[MAX_PATH];

	sprintf_s( szFileName, sizeof(szFileName), "%s\\%s%06d.%s", EVENT_DATA_DIR, MAP_EVT_FNAME_PREFIX, nMapID, MAP_EVT_FNAME_SUFFIX );

	TiXmlDocument XMLDoc;
	FILE* fp = fopen( szFileName, "rb" );
	if ( fp == NULL)
	{
		CSysError::Msg("}bvCxgǂݍݎs [%s]", szFileName );
		return FALSE;
	}
	if ( !XMLDoc.LoadFile( fp ) )
	{
		CSysError::Msg("}bvCxg p[Xs [%s]", szFileName );
		return FALSE;
	}
	fclose ( fp );

	// p[X
	TiXmlElement* elRoot = XMLDoc.RootElement();

//	p(elRoot);
	TiXmlElement* elTemp = elRoot->FirstChildElement();
	while ( elTemp )
	{
		if ( !strcmp( elTemp->Value(), "MAP_OBJ" ) )
		{
			CMapObject *pMapObj = new CMapObject();

			// p[X
			ParseMapObject( elTemp, pMapObj );

			// ʒu擾
			int iX, iY;
			GetMapObjectLocation( &iX, &iY, pMapObj->m_nID, pMapEvent );

			pMapObj->Create( iX, iY );

			// Xgɒǉ
			pMapEvent->m_lstMapObj.push_back( pMapObj );
		}
		elTemp = elTemp->NextSiblingElement();
	}
	return TRUE;
}
//-----------------------------------------------------------------------------
// GetMapObjectLocation
//-----------------------------------------------------------------------------
void CDataManager::GetMapObjectLocation( int *pX, int *pY, int nEventID, const CMapEvent *pMapEvent )
{
	for ( int iCnt = 0; iCnt < pMapEvent->m_nWidth * pMapEvent->m_nHeight; iCnt ++ )
	{
		if ( pMapEvent->m_wMapObject[iCnt] == nEventID )
		{
			*pX = iCnt % pMapEvent->m_nWidth;
			*pY = iCnt / pMapEvent->m_nWidth;
		}
	}
}
//-----------------------------------------------------------------------------
// ParseMapObject
//-----------------------------------------------------------------------------
void CDataManager::ParseMapObject( TiXmlElement* elMapObj, CMapObject *pMapObj )
{
	TiXmlElement* elTemp = elMapObj->FirstChildElement();
	while ( elTemp )
	{
		// ID
		if ( !strcmp( elTemp->Value(), "ID" ) )
		{
			pMapObj->m_nID = atoi(elTemp->GetText());
		}
		// ^Cv
		else if ( !strcmp( elTemp->Value(), "TYPE" ) )
		{
			pMapObj->m_nType = CMapObject::GetType( elTemp->GetText() );

			// ToDo:
			pMapObj->m_wPassFlag = PASS_FLAG_NO_ENTRY;
		}
		// LGID
		else if ( !strcmp( elTemp->Value(), "PICID" ) )
		{
			pMapObj->m_nPicID = atoi(elTemp->GetText());
		}
		// ړ
		else if ( !strcmp( elTemp->Value(), "MOVE" ) )
		{
			TiXmlElement* elMove = elTemp->FirstChildElement();

			while ( elMove )
			{
				// ړ^Cv
				if ( !strcmp( elMove->Value(), "TYPE" ) )
				{
					pMapObj->m_nMoveType = CMapObject::GetMoveType( elMove->GetText() );
				}
				// ړpx
				if ( !strcmp( elMove->Value(), "FREQ" ) )
				{
					pMapObj->m_nMoveFreq = CMapObject::GetMoveFreq( elMove->GetText() );
				}
				// ړx
				if ( !strcmp( elMove->Value(), "SPEED" ) )
				{
					pMapObj->m_nMoveSpeed = CMapObject::GetMoveSpeed( elMove->GetText() );
				}
				elMove = elMove->NextSiblingElement();
			}
		}
		// XNvg
		else if ( !strcmp( elTemp->Value(), "SCRIPT" ) )
		{
			TiXmlAttribute* elAttr = elTemp->FirstAttribute();
			// gK
			pMapObj->m_nScriptTriger = MAP_SCRIPT_TRIGER_NONE;
			if ( !strcmp( elAttr->Name(), "TRIGER" ) )
			{
				 pMapObj->m_nScriptTriger = CMapObject::GetScriptTriger( elAttr->Value() );
			}

			// {
			int iLen = strlen( elTemp->GetText() ) + 1;
			pMapObj->m_lpScript = new char[iLen];
			strncpy_s( pMapObj->m_lpScript, iLen, elTemp->GetText(), iLen );

		}
		elTemp = elTemp->NextSiblingElement();
	}
}

