//=============================================================================
// Game.cpp
//=============================================================================
#include "Stdinc.h"
#include "Game.h"
#include "DataManager.h"
#include "MapObject.h"

//-----------------------------------------------------------------------------
// RXgN^
//-----------------------------------------------------------------------------
CGame::CGame(void)
: m_nMode( MODE_TITLE ), m_bScriptExecuting( FALSE )
{
}
//-----------------------------------------------------------------------------
// fXgN^
//-----------------------------------------------------------------------------
CGame::~CGame(void)
{
}
//-----------------------------------------------------------------------------
// GetInstance
//-----------------------------------------------------------------------------
CGame *CGame::GetInstance( void )
{
	static CGame m_Instance;
	return &m_Instance;
}
//-----------------------------------------------------------------------------
// Initialize
//-----------------------------------------------------------------------------
BOOL CGame::Initialize( void )
{
	BOOL bRet = TRUE;

	// LN^Gǂݍ
	bRet &= CDataManager::GetInstance()->LoadCharacterGraph();

	// }bv
	bRet &= m_Map.Initialize();
	bRet &= m_Map.Load( 2 );	// ToDo:

	// EBhE쐬
	m_wndMsg.Create(  20/*X*/, 5/*Y*/, 280/**/, 72/**/ );
	m_wndMsg.ShowWindow( FALSE );

	// For debug
	//m_wndMsg.ShowWindow( TRUE );
	//m_wndMsg.SetMessage("eXg1s\t\neXg\t2s\neXg3s\neXg4s");

	m_lstDrawObj.push_back( &m_wndMsg );

	// ToDo:
	DEBUG_init();

	return bRet;
}
//-----------------------------------------------------------------------------
// Finalize
//-----------------------------------------------------------------------------
void CGame::Finalize( void )
{
	// LN^G폜
	CDataManager::GetInstance()->DeleteCharacterGraph();
}
//-----------------------------------------------------------------------------
// FrameMove
//-----------------------------------------------------------------------------
void CGame::FrameMove( void )
{
	// ANeBuł΃L[`FbN
	if ( GetActiveFlag() )
	{
		m_Mani.CheckKeyState();
	}

	switch ( m_nMode )
	{
	case MODE_MAP:	// n}ړ[h
		m_wndMsg.FrameMove( &m_Mani );
		if ( !m_wndMsg.IsShow() )
		{	// EBhE\Ȃn}͏Ȃ
			FrameMove_ModeMap();
		}
		break;

	case MODE_EVENT:	// Cxgs[h
		m_wndMsg.FrameMove( &m_Mani, &m_ScrMan );
		if ( m_wndMsg.IsWaitMessage() ||					// ̃bZ[W҂ ܂
		  ( !m_wndMsg.IsShow() && !m_bScriptExecuting ) )	// EBhE\łȂAXNvgsłȂ
		{
			CScriptOrder *pOrder = m_ScrMan.GetNext();
			if ( pOrder )
			{	// s
				ExecOrder( pOrder );
			}
			else
			{	// I[_[Ȃ
				m_nMode = m_ScrMan.GetLastGameMode();
			}
		}
		break;
	default:
		break;
	}
}
//-----------------------------------------------------------------------------
// Render
//-----------------------------------------------------------------------------
void CGame::FrameMove_ModeMap( void )
{
	// _bV
	m_Player.SetDash( m_Mani.IsKeyPushed( KEY_INPUT_Z ) );

	//  L
	m_Player.FrameMove( &m_Mani, &m_Map );

	//  }bv
	if ( !m_Map.IsScrolling() )
	{	// XN[̓L[Ȃ
		if ( m_Mani.IsKeyPushed( KEY_INPUT_UP ) )
		{
			m_Map.MoveOffsetTile( 0, -1, m_Player.GetScrollSpeed() );
		}
		else if ( m_Mani.IsKeyPushed( KEY_INPUT_DOWN ) )
		{
			m_Map.MoveOffsetTile( 0, 1, m_Player.GetScrollSpeed() );
		}
		else if ( m_Mani.IsKeyPushed( KEY_INPUT_LEFT ) )
		{
			m_Map.MoveOffsetTile( -1, 0, m_Player.GetScrollSpeed() );
		}
		else if ( m_Mani.IsKeyPushed( KEY_INPUT_RIGHT ) )
		{
			m_Map.MoveOffsetTile( 1, 0, m_Player.GetScrollSpeed() );
		}
	}
	m_Map.FrameMove( m_Player.GetScrollSpeed() );

}

//-----------------------------------------------------------------------------
// Render
//-----------------------------------------------------------------------------
void CGame::Render( void )
{
	using namespace std;

	// MAP 
	m_Map.RenderLayer(LAYER_0);

	// MAP 
	m_Map.RenderLayer(LAYER_1, &m_Player );

	// MAP 
	m_Map.RenderLayer(LAYER_2);

	// EBhE`
	list<CDrawable *>::iterator it = m_lstDrawObj.begin();
	while( it != m_lstDrawObj.end() )
	{
		(*it)->Render();
		++it;
	}

	int iX,iY;
	m_Map.GetCenterPos( &iX, &iY );
	DrawFormatString( 0 , 15 , GetColor( 255,255,255) , "[%d:%d]" , iX, iY );

}
//-----------------------------------------------------------------------------
// DEBUG_init
//-----------------------------------------------------------------------------
void CGame::DEBUG_init( void )
{
	m_nMode = MODE_MAP;

	// L쐬
	m_Player.Create( 1, 1 );
	m_Player.SetPosTile( SCREEN_CENTER_TILE_X, SCREEN_CENTER_TILE_Y, DIRECTION_U );

	m_Map.SetCenterPos( 2, 7 );
}
//-----------------------------------------------------------------------------
// StartScript
//-----------------------------------------------------------------------------
void CGame::StartScript( CMapObject *pMapObj )
{
	if ( !m_bScriptExecuting )
	{
		CScriptOrder *pOrder = m_ScrMan.Start( pMapObj, m_nMode );
		if ( pOrder )
		{
			// [hύX
			m_nMode = MODE_EVENT;

			// s
			ExecOrder( pOrder );
		}
	}
}
//-----------------------------------------------------------------------------
// ExecOrder
//-----------------------------------------------------------------------------
void CGame::ExecOrder( CScriptOrder *pOrder )
{
	//m_bScriptExecuting = TRUE;

	m_bScriptExecuting = TRUE;	// t[܂ł܂閽߂̏ꍇON

	switch ( pOrder->nID )
	{
	case SCRIPT_ORDER_SPEECH:
		// v[̕
		ChangeDirToPlayer( m_ScrMan.GetTargetMapObject() );

		m_wndMsg.SetMessage( pOrder->pParam3 );
		m_wndMsg.ShowWindow( TRUE );
		m_bScriptExecuting = FALSE;
		break;

	case SCRIPT_ORDER_TALKING:
		// v[̕
		ChangeDirToPlayer( m_ScrMan.GetTargetMapObject() );

		m_wndMsg.SetMessage( pOrder->pParam3 );
		m_wndMsg.ShowWindow( TRUE );
		m_bScriptExecuting = FALSE;
		break;

	case SCRIPT_ORDER_OPEN_STORE:
		break;
	}

	delete pOrder;
}
//-----------------------------------------------------------------------------
// ChangeDirToPlayer
//-----------------------------------------------------------------------------
void CGame::ChangeDirToPlayer( CMapObject *pMapObj )
{
	int nDirection;

	// v[Ɣ΂
	switch ( m_Player.GetDirection() )
	{
	case DIRECTION_D:
		nDirection = DIRECTION_U;
		break;
	case DIRECTION_U:
		nDirection = DIRECTION_D;
		break;
	case DIRECTION_L:
		nDirection = DIRECTION_R;
		break;
	case DIRECTION_R:
		nDirection = DIRECTION_L;
		break;
	default:
		nDirection = DIRECTION_D;
		break;
	}

	pMapObj->SetDirection( nDirection );
}
