//=============================================================================
// MapManager.cpp
//=============================================================================
#include "Stdinc.h"
#include "MapManager.h"
#include "MapObject.h"
#include "DataManager.h"
#include "Game.h"

//-----------------------------------------------------------------------------
// RXgN^
//-----------------------------------------------------------------------------
CMapManager::CMapManager(void)
	: m_nWidth( 0 ), m_nHeight( 0 ),
	m_nCenterPosX( 0 ), m_nCenterPosY( 0 ), m_nDrawOffsetX( 0 ), m_nDrawOffsetY( 0 ),
	m_bLoopAtEdge( FALSE ), m_nScrollSpeed( SCROLL_SPEED_WALK )
{
	m_wPassFlag = NULL;
	RtlZeroMemory( m_Chip, sizeof(m_Chip) );

}

//-----------------------------------------------------------------------------
// fXgN^
//-----------------------------------------------------------------------------
CMapManager::~CMapManager(void)
{
	for ( int iCnt = 0; iCnt < LAYER_CHIP_MAX; iCnt ++ )
	{
		if ( m_Chip[iCnt] )
		{
			delete [] m_Chip[iCnt];
		}
	}

	if ( m_wPassFlag )
	{
		delete [] m_wPassFlag;
	}
	// }bv`bvG폜
	CDataManager::GetInstance()->DeleteMapChipGraph();
}

//-----------------------------------------------------------------------------
// Initialize
// 
//-----------------------------------------------------------------------------
BOOL CMapManager::Initialize( void )
{
	BOOL bRet = TRUE;
	// S}bv`bvG ǂݍ
	bRet &= CDataManager::GetInstance()->LoadMapChipGraph( m_mapChipGraphInfo );

	return bRet;
}

//-----------------------------------------------------------------------------
// Load
// }bv[h
//-----------------------------------------------------------------------------
BOOL CMapManager::Load( int nID )
{
	m_nID = nID;

	BOOL bRet = TRUE;
	// wIDMAP ǂݍ
	bRet &= CDataManager::GetInstance()->LoadMap( nID, this );

	// wIDMAPCxg ǂݍ
	m_MapEvent.Clear();
	bRet &= CDataManager::GetInstance()->LoadMapEvent( nID, &m_MapEvent );

	return bRet;
}
//-----------------------------------------------------------------------------
// FrameMove
//-----------------------------------------------------------------------------
void CMapManager::FrameMove( int nScrollSpeed  )
{
	// }bvXN[
	ScrollMap( nScrollSpeed );

	// }bvIuWFNg
	m_MapEvent.FrameMove( this );
}

//-----------------------------------------------------------------------------
// ScrollMap
//-----------------------------------------------------------------------------
void CMapManager::ScrollMap( int nScrollSpeed )
{
	m_nScrollSpeed = nScrollSpeed;
	int nScrollPixel = GetScrollPixel(m_nScrollSpeed);

	// X
	if ( m_nDrawOffsetX < 0 )
	{
		m_nDrawOffsetX += nScrollPixel;
		if ( 0 < m_nDrawOffsetX )
		{
			m_nDrawOffsetX = 0;
		}
	}
	else if ( 0 < m_nDrawOffsetX )
	{
		m_nDrawOffsetX -= nScrollPixel;
		if ( m_nDrawOffsetX < 0 )
		{
			m_nDrawOffsetX = 0;
		}
	}
	// Y
	if ( m_nDrawOffsetY < 0 )
	{
		m_nDrawOffsetY += nScrollPixel;
		if ( 0 < m_nDrawOffsetY )
		{
			m_nDrawOffsetY = 0;
		}
	}
	else if ( 0 < m_nDrawOffsetY )
	{
		m_nDrawOffsetY -= nScrollPixel;
		if ( m_nDrawOffsetY < 0 )
		{
			m_nDrawOffsetY = 0;
		}
	}
}

//-----------------------------------------------------------------------------
// Render
//-----------------------------------------------------------------------------
void CMapManager::Render( void )
{
	for ( int iLayer = 0; iLayer < LAYER_CHIP_MAX; iLayer ++ )
	{
		RenderLayer( iLayer );
	}
}
//-----------------------------------------------------------------------------
// RenderLayer
// C̕`
//-----------------------------------------------------------------------------
void CMapManager::RenderLayer( int nLayer, CCharacterObject *pPlayer /* == NULL */ )
{
	using namespace std;

	int iStartX = m_nCenterPosX - SCREEN_CENTER_TILE_X;
	int iStartY = m_nCenterPosY - SCREEN_CENTER_TILE_Y;
	//if ( iStartX < 0 )		iStartX = 0;	// ToDo: GbW[v
	//if ( iStartY < 0 )		iStartY = 0;


	int nHandle;
	map<int, int>::iterator itlMap;
	itlMap = m_mapChipGraphInfo.find( m_nChipID[nLayer] );
	if ( itlMap != m_mapChipGraphInfo.end() )
	{
		nHandle = itlMap->second;
	}
	for ( int iCntY = iStartY - 1; iCntY < iStartY + TILE_COUNT_Y+1; iCntY ++ )
	{
		if ( m_nHeight <= iCntY )
			continue;	// ToDo: GbW[v

		for ( int iCntX = iStartX - 1; iCntX < iStartX + TILE_COUNT_X+1; iCntX ++ )
		{
			if ( m_nWidth <= iCntX )
				continue;	// ToDo: GbW[v

			if ( iCntX < 0 || iCntY < 0 )
			{	// ToDo: GbW[v

			}
			else
			{
				// `bv`
				DrawRectGraph( GetDrawPosX(iCntX - iStartX) + m_nDrawOffsetX, GetDrawPosY(iCntY - iStartY) + m_nDrawOffsetY,			// `W
								GetChipPosX(m_Chip[nLayer][iCntY*m_nWidth+iCntX]), GetChipPosY(m_Chip[nLayer][iCntY*m_nWidth+iCntX]),	// `悵`̍W
								MAP_CHIP_SIZE, MAP_CHIP_SIZE,
								nHandle, TRUE, FALSE );

				// C1̎ IuWFNgƃv[`
				if ( nLayer == LAYER_1 )
				{
					// IuWFNg`
					m_MapEvent.Render( iCntX, iCntY, iStartX, iStartY, m_nDrawOffsetX, m_nDrawOffsetY );	// `ʒun
					// v[`
					if ( pPlayer && iCntX == m_nCenterPosX && iCntY == m_nCenterPosY )
					{
						pPlayer->Render();
					}
				}
			}
		}
	}
}

//-----------------------------------------------------------------------------
// GetDrawPosX
//-----------------------------------------------------------------------------
int CMapManager::GetDrawPosX( int iTilePosX ) const
{
	int iPosX = iTilePosX * MAP_CHIP_SIZE - (MAP_CHIP_SIZE/2);
	return iPosX;
}
//-----------------------------------------------------------------------------
// GetDrawPosY
//-----------------------------------------------------------------------------
int CMapManager::GetDrawPosY( int iTilePosY ) const
{
	int iPosY = iTilePosY * MAP_CHIP_SIZE;
	return iPosY;
}
//-----------------------------------------------------------------------------
// GetChipPosX
//-----------------------------------------------------------------------------
int CMapManager::GetChipPosX( int iTileID ) const
{
	int iPosX = (iTileID % 0x100) * MAP_CHIP_SIZE;
	return iPosX;
}

//-----------------------------------------------------------------------------
// GetChipPosY
//-----------------------------------------------------------------------------
int CMapManager::GetChipPosY( int iTileID ) const
{
	int iPosY = (iTileID / 0x100) * MAP_CHIP_SIZE;
	return iPosY;
}
//-----------------------------------------------------------------------------
// GetScrollPixel
//-----------------------------------------------------------------------------
int CMapManager::GetScrollPixel( int nScrollSpeed ) const
{
	switch( nScrollSpeed )
	{
	case SCROLL_SPEED_WALK:			// 
		return 1;
	case SCROLL_SPEED_DASH:			// 
		return 2;
	case SCROLL_SPEED_CHOKOBO:		// `R{
		return 4;
	default:
		break;
	}

	return 1;
}
//-----------------------------------------------------------------------------
// MoveOffsetTile
// ^Cł̃ItZbgړ
//-----------------------------------------------------------------------------
BOOL CMapManager::MoveOffsetTile( int nOffsetX, int nOffsetY, int nSpeed )
{
	int nNewCenterPosX = m_nCenterPosX + nOffsetX;
	int nNewCenterPosY = m_nCenterPosY + nOffsetY;

	// Map[`FbN
	if ( ! m_bLoopAtEdge )
	{
		// E[`FbN
		if ( nNewCenterPosX < SCROLL_STOP_TILE_MARGINE_H || m_nWidth  - SCROLL_STOP_TILE_MARGINE_H <= nNewCenterPosX )
		{
			return FALSE;	// XN[Ȃ
		}
		// ㉺[`FbN
		if ( nNewCenterPosY < SCROLL_STOP_TILE_MARGINE_V || m_nHeight - SCROLL_STOP_TILE_MARGINE_V <= nNewCenterPosY )
		{
			return FALSE;	// XN[Ȃ
		}
	}
	else
	{
		// E[`FbN
		if ( nNewCenterPosX < 0 )
		{
			nNewCenterPosX = m_nWidth - 1;
		}
		else if ( m_nWidth <= nNewCenterPosX )
		{
			nNewCenterPosX = 0;
		}
		// ㉺[`FbN
		if ( nNewCenterPosY < 0 )
		{
			nNewCenterPosY = m_nHeight - 1;
		}
		else if ( m_nHeight <= nNewCenterPosY )
		{
			nNewCenterPosY = 0;
		}
	}

	// ʍstO`FbN
	if ( HitTestTile( nNewCenterPosX, nNewCenterPosY ) == PASS_FLAG_NO_ENTRY )
	{
		return FALSE;	// XN[Ȃ
	}

	m_nCenterPosX = nNewCenterPosX;
	m_nCenterPosY = nNewCenterPosY;

	m_nDrawOffsetX = nOffsetX * MAP_CHIP_SIZE;
	m_nDrawOffsetY = nOffsetY * MAP_CHIP_SIZE;
	m_nScrollSpeed = nSpeed;

	return TRUE;
}
//-----------------------------------------------------------------------------
// [static] GetNewLoc
//-----------------------------------------------------------------------------
BOOL CMapManager::GetNewLoc( int *pX, int *pY, int nLocX, int nLocY, int nDirection )
{
	*pX = nLocX;
	*pY = nLocY;

	switch ( nDirection )
	{
	case DIRECTION_U:
		(*pY) --;
		break;
	case DIRECTION_D:
		(*pY) ++;
		break;
	case DIRECTION_L:
		(*pX) --;
		break;
	case DIRECTION_R:
		(*pX) ++;
		break;
	default:
		break;
	}

	// ͈̓`FbN
	int nMapWidth  = CGame::GetInstance()->GetMapMan()->GetWidth();
	int nMapHeight = CGame::GetInstance()->GetMapMan()->GetWidth();
	if ( *pX < 0 || *pY < 0 || nMapWidth <= *pX || nMapHeight <= *pY )
	{	// ToDo: GbW[v
		*pX = nLocX;
		*pY = nLocY;
		return FALSE;
	}

	return TRUE;


}
//-----------------------------------------------------------------------------
// HitTestTile
// ^C(΍W)ł̃qbgeXg
//-----------------------------------------------------------------------------
WORD CMapManager::HitTestTile( int nX, int nY )
{
	int nIndex = nY * m_nWidth + nX;

	// ToDo: GbW[v
	if ( nX < 0 || nY < 0 || m_nWidth <= nX || m_nHeight <= nY )
		return PASS_FLAG_NO_ENTRY;

	// ^C`FbN
	WORD wRet = m_wPassFlag[nIndex];

	CMapObject *pMapObj = m_MapEvent.GetMapObject( nX, nY );
	// }bvIuWFNgΒʍstO㏑
	if ( pMapObj )
	{
		wRet = pMapObj->GetPassFlag();
	}

	// l
	if ( nX == m_nCenterPosX && nY == m_nCenterPosY )
	{
		wRet = PASS_FLAG_NO_ENTRY;
	}

	return wRet;
}

//-----------------------------------------------------------------------------
// NotifyDecideKey
// L[ʒm
//-----------------------------------------------------------------------------
void CMapManager::NotifyDecideKey( int iDirection )
{

	int iPosX;
	int iPosY;
	if ( !GetNewLoc( &iPosX, &iPosY, m_nCenterPosX, m_nCenterPosY, iDirection ) )
		return;	// }bv̈

	// ToDo: ^̃IuWFNg

	// ẴIuWFNg`FbN
	CMapObject *pMapObj = m_MapEvent.GetMapObject(iPosX,iPosY);
	if ( !pMapObj )
	{
		// Ȃ
		return;
	}

	// {^
	if ( pMapObj->GetScriptTriger() == MAP_SCRIPT_TRIGER_BUTTON )
	{
		CGame::GetInstance()->StartScript( pMapObj );
	}
}
